package FuncDialog;

import java.awt.Button;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.SwingConstants;
import javax.swing.border.BevelBorder;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import Layers.Layer;

public class LayerFuncDialog extends JDialog implements ItemListener, CaretListener, ChangeListener, ActionListener {

	private static final long serialVersionUID = 1L;

	private JPanel westPanel, centerPanel, eastPanel;
	private FlowLayout flowLayout, flowLayout2;
	private Layer layer;
	private JCheckBox check_gray, check_invert;
	private JCheckBox check_posterize , check_color, check_grad;
	private Box box;
	
	private JPanel pn_opacity, pn_color, pn_gradation;
	private Button bt_color, bt_gradFront, bt_gradRear;
	private JButton bt_merge;
	private JLabel lb_color, lb_opacity, lb_gradation, lb_gradAngle;
	private JSlider sldr_opacity, sldr_gradAngle;
	private JFormattedTextField txtf_opacity, txtf_gradAngle;
	private Box myBox;
	
	private JColorChooser colorChooser; 
	
	@SuppressWarnings("static-access")
	
	public LayerFuncDialog(Layer layer){
		this.layer = layer;
		
		flowLayout = new FlowLayout(FlowLayout.LEFT, 5, 5);
		flowLayout2 = new FlowLayout(FlowLayout.LEFT, 0, 0);
		
		this.setLayout(flowLayout);
		this.setSize(new Dimension(450, 330));
		this.setLocation(new Point(600, 200));
		this.setVisible(true);
		
		colorChooser = new JColorChooser();
		
		box = new Box(BoxLayout.Y_AXIS);
		myBox = new Box(BoxLayout.Y_AXIS);
		
		centerPanel = new JPanel();
		westPanel = new JPanel();
		eastPanel = new JPanel();
				
		westPanel.setLayout(flowLayout);
		westPanel.setPreferredSize(new Dimension(95, 285));
		westPanel.setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		westPanel.setMaximumSize(new Dimension(100, 285));
		centerPanel.setPreferredSize(new Dimension(250, 285));
		centerPanel.setBorder(BorderFactory.createTitledBorder("Effect"));
		eastPanel.setPreferredSize(new Dimension(99, 285));
				
		this.add(westPanel);
		this.add(centerPanel);
		this.add(eastPanel);
		
		check_gray = new JCheckBox("Gray"); 
		check_invert  = new JCheckBox("Invert");
		check_posterize  = new JCheckBox("Posterize");
		check_color = new JCheckBox("Color");
		check_grad  = new JCheckBox("Gradation");
		
		box.add(box.createRigidArea(new Dimension(0, 5)));
		box.add(new JSeparator());
		box.add(box.createRigidArea(new Dimension(0, 5)));
		box.add(check_color);
		box.add(check_grad);
		box.add(check_gray);
		box.add(check_invert);
		box.add(check_posterize);
		box.add(box.createRigidArea(new Dimension(0, 5)));
		box.add(new JSeparator());
		box.add(box.createRigidArea(new Dimension(0, 5)));
				
		westPanel.add(box);
		check_gray.addItemListener(this);
		check_invert.addItemListener(this);
		check_posterize.addItemListener(this);
		check_color.addItemListener(this);
		check_grad.addItemListener(this);
		
		if(this.layer.grad){
			check_grad.setSelected(true);
		}
		if(this.layer.filled){
			check_color.setSelected(true);
		}
		centerPanel.setLayout(flowLayout2);
		
		pn_opacity = new JPanel();
		pn_opacity.setPreferredSize(new Dimension(238, 55));
		pn_opacity.setLayout(null);
		pn_opacity.setBorder(BorderFactory.createTitledBorder("Opacity"));
		
		pn_color = new JPanel();
		pn_color.setPreferredSize(new Dimension(238, 80));
		pn_color.setLayout(null);
		pn_color.setBorder(BorderFactory.createTitledBorder("Color"));
		
		pn_gradation = new JPanel();
		pn_gradation.setPreferredSize(new Dimension(238, 120));
		pn_gradation.setLayout(null);
		pn_gradation.setBorder(BorderFactory.createTitledBorder("Gradation"));
				
		bt_color = new Button();
		bt_color.setBackground(layer.color);
		bt_color.addActionListener(this);
		
		bt_gradFront = new Button();
		bt_gradFront.addActionListener(this);
		bt_gradFront.setBackground(this.layer.startColor);
		bt_gradRear = new Button();
		bt_gradRear.addActionListener(this);
		bt_gradRear.setBackground(this.layer.endColor);
		
		bt_merge = new JButton("Merge");
		bt_merge.setBorder(BorderFactory.createRaisedBevelBorder());
		bt_merge.addActionListener(this);
		bt_merge.setMaximumSize(new Dimension(80,40));
		this.box.add(bt_merge);
		
		lb_opacity = new JLabel("Opacity:");			
		lb_color = new JLabel("Color Select");
		lb_gradation = new JLabel("Gradation Color");
		lb_gradAngle = new JLabel("Angle:", SwingConstants.RIGHT);
			
		sldr_opacity = new JSlider();
		sldr_opacity.addChangeListener(this);
		//int result = (int)(((float)10 - (float)layer.opacity) * (float)10);
		//sldr_opacity.setValue(0);
		sldr_gradAngle = new JSlider(SwingConstants.HORIZONTAL, 0, 360, layer.gradTheta);
		sldr_gradAngle.addChangeListener(this);
		sldr_gradAngle.setValue(this.layer.gradTheta);
		
		txtf_opacity = new JFormattedTextField();
		txtf_opacity.setHorizontalAlignment(SwingConstants.RIGHT);
		txtf_opacity.addCaretListener(this);
		
		txtf_gradAngle = new JFormattedTextField(layer.gradTheta);	
		txtf_gradAngle.setHorizontalAlignment(SwingConstants.RIGHT);
		txtf_gradAngle.addCaretListener(this);
		
		lb_opacity.setBounds(new Rectangle(15, 20, 50, 20));
		pn_opacity.add(lb_opacity);
		
		sldr_opacity.setBounds(new Rectangle(65, 20, 120, 20));
		pn_opacity.add(sldr_opacity);
		sldr_opacity.setMaximum(10);
		sldr_opacity.setValue(0);
		
		txtf_opacity.setBounds(new Rectangle(185, 20, 40, 20));
		pn_opacity.add(txtf_opacity);
		
		bt_color.setBounds(new Rectangle(15, 25, 40, 40));
		pn_color.add(bt_color);
		
		lb_color.setBounds(new Rectangle(65, 35, 70, 20));
		pn_color.add(lb_color);
		
		bt_gradFront.setBounds(new Rectangle(15, 25, 40, 40));
		pn_gradation.add(bt_gradFront);
		
		bt_gradRear.setBounds(new Rectangle(30, 40, 40, 40));
		pn_gradation.add(bt_gradRear);
	
		lb_gradation.setBounds(new Rectangle(80, 40, 90, 20));
		pn_gradation.add(lb_gradation);
		
		lb_gradAngle.setBounds(new Rectangle(15, 85, 50, 20));
		pn_gradation.add(lb_gradAngle);
		
		sldr_gradAngle.setBounds(new Rectangle(65, 85, 120, 20));
		pn_gradation.add(sldr_gradAngle);
		
		txtf_gradAngle.setBounds(new Rectangle(185, 85, 40, 20));
		pn_gradation.add(txtf_gradAngle);
			
		myBox.add(pn_opacity);
		myBox.add(pn_color);
		myBox.add(pn_gradation);
		centerPanel.add(myBox);
		
		this.requestFocus();
		this.requestFocusInWindow();
	}
	
	public void itemStateChanged(ItemEvent e) {
		Object o = e.getSource();
		int type = e.getStateChange();
	
		if(o == check_invert){
			if(type == ItemEvent.SELECTED){
				layer.invert = true;
				layer.canvasPanel.repaint();
			}else{
				layer.invert = false;
				layer.canvasPanel.repaint();
			}
		}else if(o == check_posterize){
			if(type == ItemEvent.SELECTED){
				layer.posterize = true;
				layer.canvasPanel.repaint();
			}else{
				layer.posterize = false;
				layer.canvasPanel.repaint();
			}
		}else if(o == check_gray){
			if(type == ItemEvent.SELECTED){
				layer.gray = true;
				layer.canvasPanel.repaint();
			}else{
				layer.gray = false;
				layer.canvasPanel.repaint();
			}
		}else if(o == check_grad){
			if(type == ItemEvent.SELECTED){
				layer.grad = true;
				layer.canvasPanel.repaint();
			}else{
				layer.grad = false;
				layer.canvasPanel.repaint();
			}
		}else if(o == check_color){
			if(type == ItemEvent.SELECTED){
				layer.filled = true;
				layer.canvasPanel.repaint();
			}else{
				layer.filled = false;
				layer.canvasPanel.repaint();
			}
		}
		
	}

	public void caretUpdate(CaretEvent e) {
	
		if(e.getSource().equals(txtf_opacity)) {
			try{
				//int temp = Integer.parseInt(txtf_opacity.getText());
				//sldr_opacity.setValue(temp);
				
			}catch(Exception err) {}
			
		}else if(e.getSource().equals(txtf_gradAngle)) {
			try{
				int temp = Integer.parseInt(txtf_gradAngle.getText());
				sldr_gradAngle.setValue(temp);
				
			}catch(Exception err) {}
			
		}		
	}

	public void stateChanged(ChangeEvent e) {
		
		if(e.getSource().equals(sldr_opacity)) {
			
			int value = sldr_opacity.getValue();	
			float result = ((float)10 - (float)value) / (float)10;
			this.layer.opacity = result;
			this.layer.canvasPanel.repaint();
			this.layer.canvasPanel.updateElement();
			txtf_opacity.setText("" + result);
		}else if(e.getSource().equals(sldr_gradAngle)) {
			txtf_gradAngle.setText("" + sldr_gradAngle.getValue());
			int value = sldr_gradAngle.getValue();
			this.layer.gradTheta = value;
			this.layer.canvasPanel.repaint();
			this.layer.canvasPanel.updateElement();
		}		
	}

	@SuppressWarnings("static-access")
	public void actionPerformed(ActionEvent e) {
		Object o = e.getSource();
		
		if(o == bt_color) {
			Color color= colorChooser.showDialog(this, "Color Select", layer.color);
			bt_color.setBackground(color);
			this.layer.color = color;
			this.layer.canvasPanel.repaint();
			this.layer.canvasPanel.updateElement();
		}
		else if(o == bt_gradFront) {
			Color color=colorChooser.showDialog(this, "Color Select", this.layer.startColor);
			bt_gradFront.setBackground(color);
			this.layer.startColor = color;
			this.layer.canvasPanel.repaint();
			this.layer.canvasPanel.updateElement();
		}
		else if(o == bt_gradRear) {
			Color color=colorChooser.showDialog(this, "Color Select", layer.endColor);
			bt_gradRear.setBackground(color);
			this.layer.endColor = color;
			this.layer.canvasPanel.repaint();
			this.layer.canvasPanel.updateElement();
		}else if(o == bt_merge) {
			this.layer.merge();
		}
		
	}
}
