package Layers;

import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ColorConvertOp;
import java.awt.image.LookupOp;
import java.awt.image.Raster;
import java.awt.image.ShortLookupTable;
import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;

import panels.CanvasPanel;
import utils.DrawStateSingleton;

import Shapes.ElementImage;
import Shapes.ParentElement;

public class Layer implements Serializable{
	
	private static final long serialVersionUID = 1L;
	public BufferedImage buffer;
	public boolean isVisible = true;
	private BufferedImage backup;
	private Raster raster;
	public Vector<ParentElement> vector;
	public CanvasPanel canvasPanel;
	private Graphics2D g;
	private Hashtable<String, BufferedImageOp> hash;
	GradientPaint gradient;
	public boolean grad ,filled = false, textured;
	public float gradOpacity;
	public float opacity = 1.0f;
	public float colorOpacity;
	public int gradTheta =0;
	public  Color startColor, endColor;
	public  Color color;//
	public boolean blur= false,sharpen= false,edge= false,gray = false,
	posterize = false,invert = false;
	
	public Layer(CanvasPanel canvasPanel){
		this.canvasPanel = canvasPanel;
		vector = new Vector<ParentElement>();
		hash  = new Hashtable<String, BufferedImageOp>();
		startColor = new Color(255,255,255);
		endColor = new Color(0,0,0);
		color = new Color(200,200,200);
		this.createLookups();
		this.createColorOps();
		
	}
	
	public void addElement(ParentElement element){
		this.vector.add(element);
	}
	
	public ParentElement getLastElement(){
		return this.vector.lastElement();
	}
	
	private void createColorOps() {
		hash.put("Grayscale", new ColorConvertOp(ColorSpace
	        .getInstance(ColorSpace.CS_GRAY), null));
	}
	
	private void createLookups() {
	    short[] brighten = new short[256];
	    short[] betterBrighten = new short[256];
	    short[] posterize = new short[256];
	    short[] invert = new short[256];
	    short[] straight = new short[256];
	    short[] zero = new short[256];

	    for (int i = 0; i < 256; i++) {
	      brighten[i] = (short) (128 + i / 2);
	      betterBrighten[i] = (short) (Math.sqrt((double) i / 255.0) * 255.0);
	      posterize[i] = (short) (i - (i % 32));
	      invert[i] = (short) (255 - i);
	      straight[i] = (short) i;
	      zero[i] = (short) 0;
	    }
	    
	    hash.put("Brighten", new LookupOp(new ShortLookupTable(0, brighten),
	        null));
	    hash.put("Better Brighten", new LookupOp(new ShortLookupTable(0,
	        betterBrighten), null));
	    hash.put("Posterize", new LookupOp(new ShortLookupTable(0, posterize),
	        null));
	    hash.put("Invert", new LookupOp(new ShortLookupTable(0, invert), null));

	    short[][] redOnly = { invert, straight, straight };
	    short[][] greenOnly = { straight, invert, straight };
	    short[][] blueOnly = { straight, straight, invert };
	    
	    hash.put("Red invert", new LookupOp(new ShortLookupTable(0, redOnly),
	        null));
	    hash.put("Green invert", new LookupOp(
	        new ShortLookupTable(0, greenOnly), null));
	    hash.put("Blue invert", new LookupOp(new ShortLookupTable(0, blueOnly),
	        null));

	    short[][] redRemove = { zero, straight, straight };
	    short[][] greenRemove = { straight, zero, straight };
	    short[][] blueRemove = { straight, straight, zero };
	    
	    hash.put("Red remove", new LookupOp(new ShortLookupTable(0, redRemove),
	        null));
	    hash.put("Green remove", new LookupOp(new ShortLookupTable(0,
	        greenRemove), null));
	    hash.put("Blue remove", new LookupOp(
	        new ShortLookupTable(0, blueRemove), null));
	}
	public void merge(){
		ElementImage imageEle = new ElementImage(
				DrawStateSingleton.getInstance()
				.getDrawableInformation(),this);
		imageEle.information.opacity = 1.0f;
		BufferedImage image = new BufferedImage(this.canvasPanel.getWidth(),this.canvasPanel.getHeight(),BufferedImage.TYPE_4BYTE_ABGR);
		imageEle.setRect(new Rectangle(0,0,this.canvasPanel.getWidth(),this.canvasPanel.getHeight()));
		
		
		Graphics2D imageG = image.createGraphics();
		for(int i= 0;i<vector.size();i++){
			if(vector.elementAt(i).isVisible){
				vector.elementAt(i).paint(imageG);
			}
		}
		imageEle.setImage(image);
		this.vector.removeAllElements();
		for(int i=canvasPanel.getStorage().getDrawableList().size();i>=1;i--){
			ParentElement element = canvasPanel.getStorage().getDrawableList().get(i-1);
			if( this == element.layer){
				canvasPanel.getStorage().getDrawableList().remove(i-1);
			}
		}
		this.vector.add(imageEle);
		canvasPanel.getStorage().getDrawableList().add(imageEle);
		canvasPanel.repaint();
		canvasPanel.updateElement();
	}
	public void paint(){
		
		if(this.buffer == null || buffer.getWidth(null) != canvasPanel.getWidth() || 
				buffer.getHeight(null) != canvasPanel.getHeight()){
			if(buffer != null){
				buffer.flush();
			}
			buffer = new BufferedImage(canvasPanel.getWidth(),canvasPanel.getHeight(),BufferedImage.TYPE_4BYTE_ABGR);
			g = (Graphics2D) buffer.getGraphics();
			backup = new BufferedImage(canvasPanel.getWidth(),canvasPanel.getHeight(),BufferedImage.TYPE_4BYTE_ABGR);
			this.raster =backup.getData();
		}	
		buffer.setData(raster);
		
		if(this.filled){
			for(int i=0;i<this.vector.size();i++){
				vector.elementAt(i).information.isFilled = true;
				vector.elementAt(i).information.fillColor = this.color;
			}
		}
		if(this.grad){ // Area  ΰ   Ⱥ .
			for(int i=0;i<this.vector.size();i++){
				vector.elementAt(i).information.setGradation(true);
				vector.elementAt(i).information.gradTheta = this.gradTheta;
				vector.elementAt(i).information.lineColor = this.startColor;
				vector.elementAt(i).information.fillColor = this.endColor;
			}
		}
		/*if(this.opacity != 1.0f){
			for(int i=0;i<this.vector.size();i++){
				vector.elementAt(i).information.opacity = this.opacity;
			}
		}*/
		for(int i= 0;i<vector.size();i++){
			if(vector.elementAt(i).isVisible){
				vector.elementAt(i).paint(g);
			}
		}
		
		if(this.posterize == true){
			BufferedImageOp imageOp = (BufferedImageOp) hash.get("Posterize");
			imageOp.filter(buffer, buffer);
		}
		if(this.gray == true){
			BufferedImageOp imageOp = (BufferedImageOp) hash.get("Grayscale");
			imageOp.filter(buffer, buffer);
		}
		if(this.invert == true){
			BufferedImageOp imageOp = (BufferedImageOp) hash.get("Invert");
			imageOp.filter(buffer, buffer);
		}
		
		/*if(this.grad){ // Area  ΰ   Ⱥ .
			Area area = new Area(this.vector.elementAt(0).getShape());
			for(int i = 0;i<this.vector.size();i++){
				if(this.vector.elementAt(i).getShape() != null){
					area.add(new Area(this.vector.elementAt(i).getShape()));
				}
			}
			g.setClip(area);
			gradient=new GradientPaint( 0 , 0,
	  				this.startColor,this.canvasPanel.getWidth(),
	  				this.canvasPanel.getHeight() ,this.endColor, false);
			g.setPaint(gradient);
			g.fillRect(0, 0, this.canvasPanel.getWidth(), this.canvasPanel.getHeight());
			g.clip(this.canvasPanel.getBounds());
		}*/
		
	}
}
