package ShapeSelect;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

import utils.DrawableInformation;


public class ElementPen extends ParentSelect{
	private static final long serialVersionUID = 1L;
	private GeneralPath gp;		// ׸ 
	private ArrayList<RectPenSelect> pointList;	//  ǥ ϴ 簢 迭	
	private Point2D guidLinePoint,archLinePoint1,archLinePoint2;	
	public ElementPen(DrawableInformation information) {
		super(information);
		gp = new GeneralPath();
		pointList = new ArrayList<RectPenSelect>();
	}
	public boolean isSelected(int x, int y){
		if(this.gp.contains((double)x, (double)y)){
			return true;
		}else{
			return false;
		}
	}
	
	public Point getStartPoint(){
		Point r = new Point();
		r.x = (int)gp.getCurrentPoint().getX();
		r.y = (int)gp.getCurrentPoint().getY();
		return r;
	}

	
	public boolean contains(float x, float y) {
		if(this.gp.contains(x,y)){
			return true;
		}else{
			return false;
		}
	}
	
	public void move(int x, int y){
		AffineTransform Tx = AffineTransform.getTranslateInstance(
				x, y);
		gp.transform(Tx);
		for(int i=0;i<pointList.size();i++){
			Rectangle rect = pointList.get(i).getBounds();
			rect.translate(x, y);
			pointList.get(i).setRect(rect);
		}
	}
	
	
	public void paint(Graphics2D g) {
		g.setColor(Color.black);
		g.setStroke(new BasicStroke(1,1,0));
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		
		if(this.isSelected()){
			drawGuidLine(g);
		}
		g.draw(gp);
	}
	private void drawGuidLine(Graphics2D g){
		//̵ ׸ ׸
		g.setBackground(Color.WHITE);
		g.setColor(Color.BLACK);
		
		for(RectPenSelect r:pointList){
			if(!r.isCurve())
				g.draw(r);
		}
		float[] dash=new float[]{10,5,5,5};
	    g.setStroke(new BasicStroke(1,0,BasicStroke.JOIN_MITER,1.0f,dash, 0));
		//̵  ׸
		if(guidLinePoint!=null){
			GeneralPath line = new GeneralPath();
			line.moveTo(gp.getCurrentPoint().getX(),gp.getCurrentPoint().getY());
			if(this.archLinePoint1 == null){
				line.lineTo(guidLinePoint.getX(), guidLinePoint.getY());
			}else{
				line.curveTo(gp.getCurrentPoint().getX(),gp.getCurrentPoint().getY(),
						archLinePoint1.getX(), archLinePoint1.getY(),
						guidLinePoint.getX(), guidLinePoint.getY()
						);
				Line2D curveGuid = new Line2D.Float(archLinePoint1,archLinePoint2);
				g.draw(curveGuid);
				g.fillRect((int)guidLinePoint.getX()-3, (int)guidLinePoint.getY()-3, 6, 6);
				g.fillRect((int)curveGuid.getX1()-3, (int)curveGuid.getY1()-3, 6, 6);
				g.fillRect((int)curveGuid.getX2()-3, (int)curveGuid.getY2()-3, 6, 6);
			}
			g.draw(line);
			
		}
	}

	@Override
	public void finishDraw(){
		super.finishDraw();
		guidLinePoint = null;
		archLinePoint1 = null;
	}
	public boolean addPath(float x,float y){
		if(gp.getCurrentPoint()==null){
			gp.moveTo(x, y);
			pointList.add(new RectPenSelect(x-3,y-3,6,6));
			return false;
		}else{
			gp.lineTo(x, y);
			if(pointList.get(0).contains(x,y)){
				gp.closePath();
				return true;
			}
			
			
			for(int i = 1;i<pointList.size();i++){
				if(pointList.get(i).contains(x,y)){
					if(!pointList.get(i).isCurve()){
						if(i!=(pointList.size()-1)&&pointList.get(i+1).isCurve()){
							pointList.remove(i);
							pointList.remove(i);
						}else{
							if(i != 0 && pointList.get(i-1).isCurve()){
								pointList.remove(i-1);
								pointList.remove(i-1);
							}
							else
								pointList.remove(i);
						}
					}
					updatePath();
					return false;
				}	
			}
			pointList.add(new RectPenSelect(x-3,y-3,6,6));
			
			return false;
		}	
	}
	
	private void updatePath() {
		gp.reset();
		gp.moveTo(pointList.get(0).getCenterX(), pointList.get(0).getCenterY());
		for(int i = 1;i<pointList.size();i++){
			//Ŀ
			if(pointList.get(i).isCurve()){
				float x = (float) pointList.get(i).getCenterX();
				float y = (float) pointList.get(i).getCenterY();
				i++;
				gp.curveTo(gp.getCurrentPoint().getX(),gp.getCurrentPoint().getY(),
						x,y,
						pointList.get(i).getCenterX(), pointList.get(i).getCenterY()
				);
			}else{
				gp.lineTo(pointList.get(i).getCenterX(), pointList.get(i).getCenterY());
			}
				
		}
		
	}
	public void setGuidLinePoint(float x, float y){
		
		this.guidLinePoint = new Point2D.Float(x,y);
	}
	
	public void setArchLinePoint(float x, float y){
		this.archLinePoint2 = new Point2D.Float(x,y);
		x = (float) -(x - guidLinePoint.getX());
		y = (float) -(y - guidLinePoint.getY());
		x = (float) (x + guidLinePoint.getX());
		y = (float) (y + guidLinePoint.getY());
		
		this.archLinePoint1 = new Point2D.Float(x,y);
	}
	public void finishArch(){
		gp.curveTo(gp.getCurrentPoint().getX(),gp.getCurrentPoint().getY(),
				archLinePoint1.getX(), archLinePoint1.getY(),
				guidLinePoint.getX(), guidLinePoint.getY()
		);
		pointList.add(new RectPenSelect((float)archLinePoint1.getX()-3,(float)archLinePoint1.getY()-3,6,6,true));
		pointList.add(new RectPenSelect((float)guidLinePoint.getX()-3,(float)guidLinePoint.getY()-3,6,6));
		this.archLinePoint1 = null;
	}
	public Point2D getCurrentPoint(){
		return gp.getCurrentPoint();
	}
	
	private class RectPenSelect extends Rectangle2D.Float{
		private static final long serialVersionUID = 1L;
		private boolean curve;
		public RectPenSelect(){
			super();
			curve = false;
		}
		public RectPenSelect(boolean curve){
			super();
			this.curve = curve;
		}
		public RectPenSelect(float f, float g, float i, float j) {
			super(f,g,i,j);
			curve = false;
		}
		public RectPenSelect(float f, float g, float i, float j,boolean curve) {
			super(f,g,i,j);
			this.curve = curve;
		}
		public boolean isCurve(){return curve;}
		public void setCurve(boolean curve){
			this.curve = curve;
		}
		
	}

	
	public Shape getShape(){
		return this.gp;
	}
	
}
