package Shapes;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;

import Layers.Layer;


import utils.DrawableInformation;

public class ElementImage extends ParentElement{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public Image image;
	public Rectangle rect;
	public BufferedImage bufferedImage = null;
	@SuppressWarnings("unused")
	private BufferedImageOp imageOp;
	public boolean isConverted = false;
	private Shape shape;
	public ElementImage(Image _image ,Rectangle rect){
		this.image = _image;
		this.rect = rect;
	}
	public ElementImage(DrawableInformation information,Layer layer){
		this.information = information;
		this.layer = layer;
	}
	public void setImageOp(BufferedImageOp imageOp){
		this.isConverted = true;
		this.imageOp = imageOp;
	}
	public void setImage(Image image){
		this.image = image;
		this.bufferedImage = new BufferedImage(image.getWidth(null),image.getHeight(null),BufferedImage.TYPE_4BYTE_ABGR);
		Graphics2D g = bufferedImage.createGraphics();
		g.drawImage(image, 0, 0, null);
	}
	public void setBufferedImage(BufferedImage _image){
		this.image = _image;
		//this.bufferedImage = new BufferedImage(image.getWidth(null),image.getHeight(null),BufferedImage.TYPE_4BYTE_ABGR);
		//Graphics2D g = bufferedImage.createGraphics();
		//g.drawImage(image, 0, 0, null);
		this.bufferedImage = _image;
	}
	public void setRect(Rectangle _rect){
		if(_rect.width <1 || _rect.height<1){
			
		}else{
			this.rect = _rect;
		}
	}
	public void setShape(Shape shape){
		this.shape = shape;
	}
	public Shape getShape(){
		if(this.shape != null){
			return this.shape;
		}else{
			return this.rect;
		}
	}
	public Point getStartPoint(){
		return new Point(this.rect.x,this.rect.y);
	}
	public void setStartPoint(Point p){
		this.rect.x = p.x;
		this.rect.y = p.y;
	}

	public int getHeight(){
		return this.rect.height;
	}
	public int getWidth(){
		return this.rect.width;
	}
	public void setHeight(int height){
		this.rect.height= height;
	}
	public void setWidth(int width){
		this.rect.width = width;
	}
	
	public void setResizePosition(Rectangle rect){
		this.rect.x = rect.x;
		this.rect.y = rect.y;
		this.rect.width = rect.width;
		this.rect.height = rect.height;
	}
	public void move(int x, int y){
		rect.translate(x, y);
	}
	
	public boolean isSelected(int x, int y){
		if(rect.intersects((double)x-10, (double)y-10, (double)10, (double)10))
		{
			return true;
		}else{
			return false;
		}
		
	}

	public boolean contains(float x, float y) {
		if(this.rect.contains(x,y)){
			return true;
		}else{
			return false;
		}
	}
	
	
	
	public void samllPaint(Graphics2D g){
		int rectx = rect.x;
		int recty = rect.y;
		AffineTransform at= g.getTransform();
		AffineTransform saveAt = (AffineTransform) at.clone();
		
		if(this.information.getCanvasSize().y<40){
			if(information.isRoatated == true){
				g.translate(this.centerX, this.centerY);
				g.rotate(Math.toRadians(this.information.getRotationTheta()));
				rect.x = -rect.width/2;
				rect.y = -rect.height/2;
			}
			//System.out.println("canvasSize :" +this.information.getCanvasSize().y);
			g.drawImage(bufferedImage, rect.x, rect.y, rect.width, rect.height,null);
		}else{
			float rate = (float)this.information.getCanvasSize().y /(float)40;
			//System.out.println("canvasSize :" +this.information.getCanvasSize().y);
			if(information.isRoatated == true){
				g.translate(this.centerX/rate, this.centerY/rate);
				g.rotate(Math.toRadians(this.information.getRotationTheta()));
				rect.x = -rect.width/2;
				rect.y = -rect.height/2;
			}
			g.scale(1/rate, 1/rate);
			g.drawImage(bufferedImage, rect.x, rect.y, rect.width, rect.height,null);
			g.draw(rect);
			
			
		}
		if(information.isRoatated == true){
			rect.x = rectx;
			rect.y = recty;
		}
		g.setTransform(saveAt);
	}
	
	public void middlePaint(Graphics2D g){
		int rectx = rect.x;
		int recty = rect.y;
		
		
		this.information.setGraphicAttribute(g);
		AffineTransform at= g.getTransform();
		AffineTransform saveAt = (AffineTransform) at.clone();
		if(this.information.getCanvasSize().y<150){
			if(information.isRoatated == true){
				g.translate(this.centerX, this.centerY);
				g.rotate(Math.toRadians(this.information.getRotationTheta()));
				rect.x = -rect.width/2;
				rect.y = -rect.height/2;
			}
			g.drawImage(bufferedImage, rect.x, rect.y, rect.width, rect.height,null);
		}else{
			float rate = (float)this.information.getCanvasSize().y/(float)150;
			if(information.isRoatated == true){
				g.translate(this.centerX/rate, this.centerY/rate);
				g.rotate(Math.toRadians(this.information.getRotationTheta()));
				rect.x = -rect.width/2;
				rect.y = -rect.height/2;
			}
			
			g.scale(1/rate, 1/rate);
			
			g.drawImage(bufferedImage, rect.x, rect.y, rect.width, rect.height,null);
			g.draw(rect);
			g.setTransform(saveAt);
		}
		if(information.isRoatated == true){
			rect.x = rectx;
			rect.y = recty;
		}
		g.setTransform(saveAt);
	}
	
	public void paint(Graphics2D g){
		this.information.setGraphicAttribute(g);
		AffineTransform at= g.getTransform();
		AffineTransform saveAt = (AffineTransform) at.clone();
		AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 
				this.information.opacity);
		g.setComposite(ac);
		if(information.isRoatated == true){
			this.centerX= (int)rect.getCenterX();
			this.centerY = (int)rect.getCenterY();
		
			g.translate(this.centerX, this.centerY);
			g.rotate(Math.toRadians(this.information.getRotationTheta()));
			rect.x = -rect.width/2;
			rect.y = -rect.height/2;
		}
		
		g.setRenderingHint(RenderingHints.KEY_INTERPOLATION,
		        RenderingHints.VALUE_INTERPOLATION_BICUBIC);
		g.drawImage(bufferedImage, rect.x, rect.y, rect.width, rect.height,null);
		if(this.isSelected()){
			g.setStroke(new BasicStroke(1,0,BasicStroke.JOIN_MITER,1.0f,dash, 0));
			g.drawRect(this.rect.x-1, this.rect.y-1,this.rect.width+2,this.rect.height+2);
			g.setStroke(new BasicStroke(this.information.strokeSize,1,0));
		}
		
		if(information.isRoatated == true){
		rect.x = this.centerX-rect.width/2;
		rect.y = this.centerY-rect.height/2;
		}
		
		
		g.setTransform(saveAt);
	}
	
	
}
