package Shapes;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.TexturePaint;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Rectangle2D;

import Layers.Layer;

import utils.DrawableInformation;

public class ElementLine extends ParentElement{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Line2D.Double line;
	private Rectangle rect;
	public ElementLine(DrawableInformation information,Layer layer){
		this.information = information;
		rect = new Rectangle(0,0,0,0);
		this.layer = layer;
	}
	public void setLine(Line2D.Double line){
		this.line = line;
		rect.x =(int)line.x1;
		rect.y = (int)line.y1;
		rect.width = (int)Math.abs(line.x2 -line.x1 );
		rect.height = (int)Math.abs(line.y2 -line.y1);
		
	}
	public Shape getShape(){
		return line;
	}
	public void setStartPoint(Point _stpoint){
		Point s = (Point)_stpoint;
		line.x1 = s.x;
		line.y1 = s.y;
		rect.x =(int)line.x1;
		rect.y = (int)line.y1;
		rect.width = (int)Math.abs(line.x2 -line.x1 );
		rect.height = (int)Math.abs(line.y2 -line.y1);
	}
	
	public void setEndPoint(Point _endpoint){
		Point s= (Point)_endpoint;
		line.x2 = s.x;
		line.y2 = s.y;
		rect.x =(int)line.x1;
		rect.y = (int)line.y1;
		rect.width = (int)Math.abs(line.x2 -line.x1 );
		rect.height = (int)Math.abs(line.y2 -line.y1);
	}
	public Point getStartPoint(){
		return new Point((int)line.x1,(int)line.y1);
	}
	
	public Point getEndPoint(){
		return new Point((int)line.x2,(int)line.y2);
	}
	public void move(int x, int y){
		line.x1 = line.x1 + x;
		line.y1 = line.y1 + y;
		line.x2 = line.x2 + x;
		line.y2 = line.y2 + y;
		
		rect.x =(int)line.x1;
		rect.y = (int)line.y1;
		rect.width = (int)Math.abs(line.x2 -line.x1 );
		rect.height = (int)Math.abs(line.y2 -line.y1);
	}
	public boolean isSelected(int x, int y){
		if(line.intersects((double)x-5, (double)y-5, (double)10, (double)10))
		{
			return true;
		}else{
			return false;
		}
		
	}
	
	public boolean contains(float x, float y) {
		
		if(this.rect.contains(x,y)){
			return true;
		}else{
			return false;
		}
	}
	
	public void samllPaint(Graphics2D g){
		AffineTransform at= g.getTransform();
		AffineTransform saveAt = (AffineTransform) at.clone();
		if(this.information.getCanvasSize().y<40){
			g.draw(line);
		}else{
			float rate = (float)this.information.getCanvasSize().y /(float)40;
			g.scale(1/rate, 1/rate);
			g.draw(line);
			g.setTransform(saveAt);
		}
	}
	
	public void middlePaint(Graphics2D g){
		this.information.setGraphicAttribute(g);
		AffineTransform at= g.getTransform();
		AffineTransform saveAt = (AffineTransform) at.clone();
		if(this.information.getCanvasSize().y<150){
			g.draw(line);
			if(information.getGradation()){
				GradientPaint grad;
		  		grad=new GradientPaint(information.getGradStart().x, information.getGradStart().y,
		  				information.lineColor,information.getGradEnd().x,
		  				information.getGradEnd().y ,information.fillColor, false);
		  		
		  		g.setPaint(grad);
			}
		}else{
			float rate = (float)this.information.getCanvasSize().y /(float)150;
			g.scale(1/rate, 1/rate);
			g.draw(line);
			if(information.getGradation()){
				GradientPaint grad;
		  		grad=new GradientPaint(information.getGradStart().x, information.getGradStart().y,
		  				information.lineColor,information.getGradEnd().x,
		  				information.getGradEnd().y ,information.fillColor, false);
		  		
		  		g.setPaint(grad);
			}
			g.setTransform(saveAt);
		}
	}
	
	public void paint(Graphics2D g){
		this.information.setGraphicAttribute(g);
		AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 
				this.information.opacity);
		g.setComposite(ac);
		if(information.getGradation()){
			GradientPaint grad;
	  		grad=new GradientPaint(information.getGradStart().x, information.getGradStart().y,
	  				information.lineColor,information.getGradEnd().x,
	  				information.getGradEnd().y ,information.fillColor, false);
	  		
	  		g.setPaint(grad);
		}
		if(this.information.getTexture()){
			if(this.texture != null){
				TexturePaint tpaint = new TexturePaint(this.texture, new Rectangle2D.Float(10,10,50,50));
				g.setPaint(tpaint);
			}
		}
		if(this.isSelected() == true){
			g.setStroke(new BasicStroke(1,0,BasicStroke.JOIN_MITER,1.0f,dash, 0));
			g.draw(this.line);
			g.setStroke(new BasicStroke(this.information.strokeSize,1,0));
		}else{
			g.draw(this.line);
		}
		
	}
	
}
