package Shapes;


import java.awt.AlphaComposite;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.TexturePaint;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.Rectangle2D;

import Layers.Layer;


import utils.DrawableInformation;

public class ElementPencil extends ParentElement{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private GeneralPath gp = null;
	
	
	public ElementPencil(DrawableInformation information,Layer layer){
		this.information = information;
		this.layer = layer;
		this.information.setFill(false);
	}
	public void setGeneralPath(GeneralPath gp){
		this.gp = gp;
	}
	public Shape getShape(){
		return gp;
	}
	public boolean isSelected(int x, int y){
		if(gp.intersects((double)x-5, (double)y-5, (double)10, (double)10))
		{
			return true;
		}else{
			return false;
		}
		
	}
	
	public boolean contains(float x, float y) {
		if(this.gp.contains(x,y)){
			return true;
		}else{
			return false;
		}
	}
	
	
	public Point getStartPoint(){
		Point r = new Point();
		r.x = (int)gp.getCurrentPoint().getX();
		r.y = (int)gp.getCurrentPoint().getY();
		return r;
	}
	
	public void move(int x, int y){
		AffineTransform Tx = AffineTransform.getTranslateInstance(
				x, y);
		gp.transform(Tx);
		
	}
	
	
	public void samllPaint(Graphics2D g){
		AffineTransform at= g.getTransform();
		AffineTransform saveAt = (AffineTransform) at.clone();
		if(this.information.getCanvasSize().y<40){
			g.draw(gp);
		}else{
			float rate = (float)this.information.getCanvasSize().y /(float)40;
			g.scale(1/rate, 1/rate);
			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
					RenderingHints.VALUE_ANTIALIAS_ON);
			g.draw(gp);
			g.setTransform(saveAt);
		}
	}
	
	public void middlePaint(Graphics2D g){
		this.information.setGraphicAttribute(g);
		AffineTransform at= g.getTransform();
		AffineTransform saveAt = (AffineTransform) at.clone();
		if(this.information.getCanvasSize().y<150){
			this.information.setGraphicAttribute(g);
			
			if(information.getGradation()){
				GradientPaint grad;
		  		grad=new GradientPaint(information.getGradStart().x, information.getGradStart().y,
		  				information.lineColor,information.getGradEnd().x,
		  				information.getGradEnd().y ,information.fillColor, false);
		  		
		  		g.setPaint(grad);
		  		g.draw(gp);
			}else{
				g.setColor(this.information.getLineColor());
				g.draw(gp);
			}
		}else{
			float rate = (float)this.information.getCanvasSize().y /(float)150;
			g.scale(1/rate, 1/rate);
			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON);
			this.information.setGraphicAttribute(g);
			
			if(information.getGradation()){
				GradientPaint grad;
		  		grad=new GradientPaint(information.getGradStart().x, information.getGradStart().y,
		  				information.lineColor,information.getGradEnd().x,
		  				information.getGradEnd().y ,information.fillColor, false);
		  		
		  		g.setPaint(grad);
		  		g.draw(gp);
			}else{
				g.setColor(this.information.getLineColor());
				g.draw(gp);
			}
			g.setTransform(saveAt);
		}
	}
	
	public void paint(Graphics2D g){
		this.information.setGraphicAttribute(g);
		AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 
				this.information.opacity);
		g.setComposite(ac);
		
		if(information.getGradation()){
			GradientPaint grad;
	  		grad=new GradientPaint(information.getGradStart().x, information.getGradStart().y,
	  				information.lineColor,information.getGradEnd().x,
	  				information.getGradEnd().y ,information.fillColor, false);
	  		
	  		g.setPaint(grad);
	  		if(this.information.isFilled){
	  			
				g.fill(gp);
				g.setColor(this.information.lineColor);
				g.draw(gp);
				if(this.information.getTexture()){
					if(this.texture != null){
						TexturePaint tpaint = new TexturePaint(this.texture, new Rectangle2D.Float(10,10,50,50));
						g.setPaint(tpaint);
						g.fill(gp);
						g.setColor(this.information.lineColor);
						g.draw(gp);
					}
				}
			}else{
				g.draw(gp);
				if(this.information.getTexture()){
					if(this.texture != null){
						TexturePaint tpaint = new TexturePaint(this.texture, new Rectangle2D.Float(10,10,50,50));
						g.setPaint(tpaint);
						g.draw(gp);
					}
				}
			}
	  		
		}else{
			g.setColor(this.information.getLineColor());
			if(this.information.isFilled){
				g.setColor(this.information.fillColor);
				
				if(this.information.getTexture()){
					if(this.texture != null){
						TexturePaint tpaint = new TexturePaint(this.texture, new Rectangle2D.Float(10,10,50,50));
						g.setPaint(tpaint);
						
					}
				}
				g.fill(gp);
				g.setColor(this.information.lineColor);
				g.draw(gp);
			}else{
				if(this.information.getTexture()){
					if(this.texture != null){
						TexturePaint tpaint = new TexturePaint(this.texture, new Rectangle2D.Float(10,10,50,50));
						g.setPaint(tpaint);
						
					}
				}
				g.draw(gp);
			}
		}
	}

	
}
