package Shapes;

import java.awt.AlphaComposite;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.TexturePaint;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;

import Layers.Layer;

import utils.DrawableInformation;

public class ElementPolygon extends ParentElement{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private Polygon polygon = null;
	
	public ElementPolygon(Polygon polygon){
		this.polygon = polygon;
	}
	public ElementPolygon(DrawableInformation information,Layer layer){
		this.information = information;
		this.layer = layer;
	}
	public void setPolygon(Polygon polygon){
		this.polygon = polygon;
	}
	public Shape getShape(){
		return this.polygon;
	}
	public boolean isSelected(int x, int y){
		if(polygon.intersects((double)x-5, (double)y-5, (double)10, (double)10))
		{
			return true;
		}else{
			return false;
		}
		
	}

	public boolean contains(float x, float y) {
		if(this.polygon.contains(x,y)){
			return true;
		}else{
			return false;
		}
	}
	
	public Point getStartPoint(){
		Point r = new Point();
		r.x = polygon.xpoints[0];
		r.y = polygon.ypoints[0];
		
		return r;
	}
	
	public void move(int x, int y){
		polygon.translate(x, y);
	}
	
	public void samllPaint(Graphics2D g){
		AffineTransform at= g.getTransform();
		AffineTransform saveAt = (AffineTransform) at.clone();
		if(this.information.getCanvasSize().y<40){
			g.draw(polygon);
		}else{
			float rate = (float)this.information.getCanvasSize().y /(float)40;
			g.scale(1/rate, 1/rate);
			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
					RenderingHints.VALUE_ANTIALIAS_ON);
			g.setColor(Color.white);
			g.fill(polygon);
			g.setColor(Color.black);
			g.draw(polygon);
			g.setTransform(saveAt);
		}
	}
	
	public void middlePaint(Graphics2D g){
		this.information.setGraphicAttribute(g);
		AffineTransform at= g.getTransform();
		AffineTransform saveAt = (AffineTransform) at.clone();
		if(this.information.getCanvasSize().y<150){
			g.draw(polygon);
			if(this.information.isFilled){
				if(information.getGradation()){
					GradientPaint grad;
			  		grad=new GradientPaint(information.getGradStart().x, information.getGradStart().y,
			  				information.lineColor,information.getGradEnd().x,
			  				information.getGradEnd().y ,information.fillColor, false);
			  		
			  		g.setPaint(grad);
					g.fill(polygon);
				}else{
					g.setColor(this.information.getFillColor());
					g.fill(this.polygon);
				}
			}
		}else{
			float rate = (float)this.information.getCanvasSize().y /(float)150;
			g.scale(1/rate, 1/rate);
			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
					RenderingHints.VALUE_ANTIALIAS_ON);
			g.draw(polygon);
			if(this.information.isFilled){
				if(information.getGradation()){
					GradientPaint grad;
			  		grad=new GradientPaint(information.getGradStart().x, information.getGradStart().y,
			  				information.lineColor,information.getGradEnd().x,
			  				information.getGradEnd().y ,information.fillColor, false);
			  		
			  		g.setPaint(grad);
					g.fill(polygon);
				}else{
					g.setColor(this.information.getFillColor());
					g.fill(this.polygon);
				}
			}
			g.setTransform(saveAt);
		}
	}
	public void paint(Graphics2D g){
		this.information.setGraphicAttribute(g);
		
		AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 
				this.information.opacity);
		g.setComposite(ac);
		if(this.isSelected()){
			g.setStroke(new BasicStroke(1,0,BasicStroke.JOIN_MITER,1.0f,dash, 0));
			g.draw(this.polygon);
			g.setStroke(new BasicStroke(this.information.strokeSize,1,0));
		}else{
			if(information.getGradation()){
				AlphaComposite c = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, this.information.gradOpacity);
				g.setComposite(c);
				
				if(this.information.gradTheta !=0) {
					g.clip(this.polygon);
					float cross = (polygon.getBounds().width)*(polygon.getBounds().width) + (polygon.getBounds().height)*(polygon.getBounds().height);
					int result = (int)Math.sqrt(cross);
					Rectangle rect2 = new Rectangle( (int)(- result/2) , (int)(- result /2) ,result,result);
					AffineTransform at2 = new AffineTransform();	
					AffineTransform saveform = g.getTransform();		
					at2.setToTranslation(polygon.getBounds().x + polygon.getBounds().width/2, polygon.getBounds().y + polygon.getBounds().height/2);
					at2.rotate(Math.toRadians(this.information.gradTheta)); 
					g.transform(at2); 
					GradientPaint grad;
			  		grad=new GradientPaint( rect2.x , rect2.y,
			  				information.lineColor,rect2.x+rect2.width,
			  				rect2.y + rect2.height ,information.fillColor, false);
			  		g.setPaint(grad);
			  		
			  		g.setTransform(saveform);
			  		g.setClip(null);
				}else{
					GradientPaint grad;
			  		grad=new GradientPaint( polygon.getBounds().x, polygon.getBounds().y,
			  				information.lineColor,polygon.getBounds().x+polygon.getBounds().width,
			  				polygon.getBounds().y+polygon.getBounds().height ,information.fillColor, false);
			  		g.setPaint(grad);
			  		
				}
			}
			if(this.information.getTexture()){
				if(this.texture != null){
					TexturePaint tpaint = new TexturePaint(this.texture, new Rectangle2D.Float(10,10,50,50));
					g.setPaint(tpaint);
				}
			}
			g.draw(this.polygon);
		}
		
		if(this.information.isFilled){
			if(information.getGradation()){
				AlphaComposite c = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, this.information.gradOpacity);
				g.setComposite(c);
				
				if(this.information.gradTheta !=0) {
					g.clip(this.polygon);
					float cross = (polygon.getBounds().width)*(polygon.getBounds().width) + (polygon.getBounds().height)*(polygon.getBounds().height);
					int result = (int)Math.sqrt(cross);
					Rectangle rect2 = new Rectangle( (int)(- result/2) , (int)(- result /2) ,result,result);
					AffineTransform at2 = new AffineTransform();	
					AffineTransform saveform = g.getTransform();		
					at2.setToTranslation(polygon.getBounds().x + polygon.getBounds().width/2, polygon.getBounds().y + polygon.getBounds().height/2);
					at2.rotate(Math.toRadians(this.information.gradTheta)); 
					g.transform(at2); 
					GradientPaint grad;
			  		grad=new GradientPaint( rect2.x , rect2.y,
			  				information.lineColor,rect2.x+rect2.width,
			  				rect2.y + rect2.height ,information.fillColor, false);
			  		g.setPaint(grad);
			  		if(this.information.getTexture()){
						if(this.texture != null){
							TexturePaint tpaint = new TexturePaint(this.texture, new Rectangle2D.Float(10,10,50,50));
							g.setPaint(tpaint);
						}
					}
			  		g.fill(rect2);
			  		
			  		g.setTransform(saveform);
			  		g.setClip(null);
				}else{
					GradientPaint grad;
			  		grad=new GradientPaint( polygon.getBounds().x, polygon.getBounds().y,
			  				information.lineColor,polygon.getBounds().x+polygon.getBounds().width,
			  				polygon.getBounds().y+polygon.getBounds().height ,information.fillColor, false);
			  		g.setPaint(grad);
			  		if(this.information.getTexture()){
						if(this.texture != null){
							TexturePaint tpaint = new TexturePaint(this.texture, new Rectangle2D.Float(10,10,50,50));
							g.setPaint(tpaint);
						}
					}
					g.fill(polygon);
				}
				
				
				
			}else{
				g.setColor(this.information.getFillColor());
				if(this.information.getTexture()){
					if(this.texture != null){
						TexturePaint tpaint = new TexturePaint(this.texture, new Rectangle2D.Float(10,10,50,50));
						g.setPaint(tpaint);
					}
				}
				g.fill(this.polygon);
			}
		}
		
	}

	
}
