package Start;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.geom.Area;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JToolBar;

import FuncDialog.imageFunDialog;
import Layers.Layer;
import Shapes.ElementFrame;
import Shapes.ElementImage;
import Shapes.ParentElement;

import panels.BrushToolPanel;
import panels.KoreaBrushToolPanel;

import subframes.CanvasFrame;
import subframes.EditFrame;
import subframes.FormFrame;
import subframes.HistoryFrame;

import toolBars.BloatTool;
import toolBars.BlurTool;
import toolBars.EditTool;
import toolBars.PuckerTool;
import toolBars.SharpenTool;
import toolBars.StampTool;
import toolBars.NorthTool;
import toolBars.WestTool;
import utils.DrawStateSingleton;

import menus.EditMenu;
import menus.FileMenu;
import menus.HelpMenu;
import menus.InfoMenu;
import menus.SelectMenu;
import menus.WindowMenu;


public class MainFrame extends JFrame{
	
	private static final long serialVersionUID = 1L;
	
	private Vector<CanvasFrame> canvasVector;	
	private JMenuBar menuBar;
	private Toolkit myToolkit;
	@SuppressWarnings("unused")
	private FlowLayout flowLayout;
	MediaTracker mt;
	
	private WindowMenu windowMenu;
	private FileMenu fileMenu;
	private EditMenu editMenu;
	private SelectMenu selectMenu;
	private HelpMenu helpMenu;
	private InfoMenu infoMenu;
	private JPanel pn_tool;
	
	private NorthTool northTool;
	private EditTool editTool;
	public WestTool westTool;
	private StampTool stampTool;
	private BloatTool bloatTool;
	private PuckerTool puckerTool;
	private BlurTool blurTool;
	private SharpenTool sharpenTool;
	
	public JDesktopPane desktopPane;
	
	public CanvasFrame currentCanvasFrame = null;
	public HistoryFrame historyFrame;
	public EditFrame editFrame;
	public FormFrame formFrame;
	public imageFunDialog imgFunFrame;
	
	private KoreaBrushToolPanel koreaBrushToolPanel;
	private BrushToolPanel brushToolPanel;
	private int addCanvasPosition =10;
	private ParentElement selectedEle = null;
	private Image healingImage = null;
	private Hashtable<String, JToolBar> hashTable;
	
	public  int stampX, stampY;
	public  int scalex, scaley;
	public BufferedImage stampImage;
	public Vector<ParentElement> frameVector;
	
	private Image[] imageArr;
	
	public MainFrame(String name){
		super(name);
		
		imageArr = new Image[11];
		mt = new MediaTracker(this);
		myToolkit = Toolkit.getDefaultToolkit();
		flowLayout = new FlowLayout(FlowLayout.LEFT, 5, 5);
		frameVector = new Vector<ParentElement>();
		canvasVector = new Vector<CanvasFrame>();
		hashTable = new Hashtable<String, JToolBar>();
		menuBar = new JMenuBar();
		fileMenu = new FileMenu("File(F)",this);
		editMenu = new EditMenu("Edit(E)");
		selectMenu = new SelectMenu("Select(S)");
		helpMenu = new HelpMenu("Help(H)");
		infoMenu = new InfoMenu("Info(I)");
				
		pn_tool = new JPanel();
		
		northTool = new NorthTool(this);
		editTool = new EditTool(this);
		westTool = new WestTool(this);
		stampTool = new StampTool(this);
		bloatTool = new BloatTool(this);
		puckerTool = new PuckerTool(this);
		blurTool = new BlurTool(this);
		sharpenTool = new SharpenTool(this);
		
		hashTable.put("edit", editTool);
		hashTable.put("stamp", stampTool);
		hashTable.put("bloat", bloatTool);
		hashTable.put("pucker", puckerTool);
		hashTable.put("blur", blurTool);
		hashTable.put("sharpen", sharpenTool);
		
		desktopPane = new JDesktopPane();
		desktopPane.setBackground(new Color(80,80,80));
		
		currentCanvasFrame = new CanvasFrame(this);
		currentCanvasFrame.setCanvasBounds(addCanvasPosition);
		canvasVector.add(currentCanvasFrame);
		currentCanvasFrame.setName(String.valueOf(canvasVector.size()));
		DrawStateSingleton.getInstance().setCurrentPanel(currentCanvasFrame.canvasPanel);
		historyFrame = new HistoryFrame(this);
		editFrame = new EditFrame(this);
		formFrame = new FormFrame(this);
		formFrame.setVisible(false);
		
		
		windowMenu = new WindowMenu("Windows(W)",this);
		
		koreaBrushToolPanel = new KoreaBrushToolPanel();
		brushToolPanel = new BrushToolPanel();
		
		this.sub();
	}
	
	public void sub(){
		this.setLayout(new BorderLayout());
		this.setJMenuBar(menuBar);
		this.setFocusable(true);

		pn_tool.setLayout(new BoxLayout(pn_tool, BoxLayout.Y_AXIS));
		editTool.setKoreaBrushToolPanel(koreaBrushToolPanel);
		editTool.setBrushToolPanel(brushToolPanel);
		
		menuBar.add(fileMenu);
		menuBar.add(editMenu);
		menuBar.add(selectMenu);
		menuBar.add(helpMenu);
		menuBar.add(infoMenu);
		menuBar.add(windowMenu);

		pn_tool.add(northTool);
		pn_tool.add(new JSeparator());
		pn_tool.add(editTool);

		this.add(BorderLayout.NORTH, pn_tool);
		this.add(BorderLayout.WEST, westTool);
				
		desktopPane.add(currentCanvasFrame);
		desktopPane.add(historyFrame);
		desktopPane.add(editFrame);
		desktopPane.add(formFrame);
	
		desktopPane.add(koreaBrushToolPanel);
		desktopPane.setComponentZOrder(koreaBrushToolPanel, 0);
		desktopPane.add(brushToolPanel);
		desktopPane.setComponentZOrder(brushToolPanel, 0);
		
		this.currentCanvasFrame.canvasPanel.updateElement();
		this.getContentPane().add(desktopPane);	
		this.makeFrame();
	}	
	public void swapToolBar(String tool){
		
		for(int i= 3 ;i>2;i--){
			pn_tool.remove(i-1);
		}
		pn_tool.add(hashTable.get(tool));
		
		
		pn_tool.validate();
		pn_tool.repaint();
		this.repaint();
	}
	
	public void addCanvasFrame(){
		CanvasFrame canvasframe = new CanvasFrame(this);
		canvasframe.setCanvasBounds(this.addCanvasPosition+=20);
		canvasVector.add(canvasframe);
		canvasframe.setName(String.valueOf(canvasVector.size()));	
		this.currentCanvasFrame = canvasframe;
		this.desktopPane.add(canvasframe);
		
		desktopPane.setComponentZOrder(canvasframe, 0);
	}
	
	public void addCanvasFrame(Image image){
		System.gc();
		System.runFinalization();
		CanvasFrame canvasframe = new CanvasFrame(this);
		canvasframe.setCanvasBounds(this.addCanvasPosition+=20 , image.getWidth(null),image.getHeight(null));
		canvasVector.add(canvasframe);
		canvasframe.setName(String.valueOf(canvasVector.size()));	
		this.currentCanvasFrame = canvasframe;
		this.desktopPane.add(canvasframe);
		
		desktopPane.setComponentZOrder(canvasframe, 0);
		
		
		DrawStateSingleton.getInstance().setCurrentPanel(canvasframe.getCanvasPanel());
		if(DrawStateSingleton.getInstance().getToolState() != null){
			DrawStateSingleton.getInstance().getToolState().setCanvas(canvasframe.canvasPanel);
		}
		ElementImage ele= new ElementImage(DrawStateSingleton.getInstance()
				.getDrawableInformation(),canvasframe.canvasPanel.currentLayer);
		ele.setRect(new Rectangle(0,0,image.getWidth(null),image.getHeight(null)));
		ele.setImage(image);
		canvasframe.canvasPanel.currentLayer.vector.add(ele);
		canvasframe.canvasPanel.getStorage().getDrawableList().add(ele);
		//System.out.println(image.getWidth(null) +" ,"+ image.getHeight(null));
		//System.out.println(canvasframe.getBounds());
		canvasframe.canvasPanel.updateElement();
	}
	
	public void removeCanvasFrame(CanvasFrame canvasframe) {
		canvasVector.remove(canvasframe);
		//System.out.println(" Ǿ.");
	}
	
	public void setCurrentCanvasFrame(CanvasFrame cf) {
		this.currentCanvasFrame = cf;
		//System.out.println(this.currentCanvasFrame.getName()+" Ŀ   ");
	}
	
	public void removeCurrentFocusFrame() {
		this.currentCanvasFrame= null;
	}
	
    public CanvasFrame getCurrentCanvasFrame() {
    	if(this.currentCanvasFrame != null) {
    		return this.currentCanvasFrame;
    	}else {
    		return null;
    	}
	}
    
    public void setCurrentCanvasFrameDrawMode(byte dmode){
    	if((this.currentCanvasFrame != null)  && (this.canvasVector.size() != 0)){
    		currentCanvasFrame.setDrawMode(dmode);
    	}else{
    		
    	}
    }

	public void setImageToCanvasFrame(Image temporeryImage) {
		if((this.currentCanvasFrame != null)  && (this.canvasVector.size() != 0)){
    		currentCanvasFrame.setImageToCanvasFrame(temporeryImage);
    	}else{
    		
    	}
		
	}
	public void addElement(ParentElement element){
		
	}
	
	public void setStampImage(int resultX , int resultY ,BufferedImage image ,int x, int y){
		this.stampX = resultX;
		this.stampY = resultY;
		this.stampImage = image;
		this.scalex = x;
		this.scaley = y;
		
	}
	public BufferedImage getStampImage(){
		//if(this.stampImage != null){
			return stampImage;
		//}
	}
	
	public Point getStampPoint(){
		return new Point(this.stampX , this.stampY);
	}
	public Point getStampScale(){
		return new Point(this.scalex , this.scaley);
	}
	public void updateElement(ArrayList<ParentElement> drawableList) {
		this.historyFrame.updateElement(drawableList,this.currentCanvasFrame.canvasPanel.layerVector);
		this.editFrame.updateElement(drawableList);
		//this.layerFrame.updateElement(this.currentCanvasFrame.canvasPanel.layerVector);
	}
	public void setSelectedElement(ParentElement ele){
		this.selectedEle = ele;
	}
	public  ParentElement getSelectedElement(){
		return this.selectedEle;
	}
	
	public void setHealingImage(Image image){
		this.healingImage = image;
	}
	
	public Image getHealingImage(){
		return this.healingImage;
	}
	public void makeFrame(){
		String name = new String("image/frame");
		for(int i=0;i<11;i++){
			String path = name+i+".jpg";
			imageArr[i] = myToolkit.getImage(path); // 400,500
			mt.addImage(imageArr[i], i);
		}
		
		try {
			mt.waitForAll();
		} catch (InterruptedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		for(int i=0;i<11;i++){
			BufferedImage buf = new BufferedImage(400,500,BufferedImage.TYPE_4BYTE_ABGR);
			Graphics2D bufg = buf.createGraphics();
			Area area = new Area(new Rectangle(0,0,buf.getWidth(),buf.getHeight()));
			area.subtract(new Area(new Rectangle(50,50,300,400)));
			bufg.clip(area);
			bufg.drawImage(imageArr[i], 0, 0,400,500, null);
			ElementFrame ele = new ElementFrame(DrawStateSingleton.getInstance()
					.getDrawableInformation(),this.currentCanvasFrame.canvasPanel.currentLayer);
			
			ele.setRect(new Rectangle(0,0,400,500));
			ele.setBufferedImage(buf);
			//ele.information.setCanvasSize(this.currentCanvasFrame.canvasPanel.getWidth(), this.currentCanvasFrame.canvasPanel.getHeight());
			this.frameVector.add(ele);
			buf.flush();
			bufg.finalize();
			
		}
		for(int i = 0;i<11;i++){
			mt.removeImage(imageArr[i]);
			imageArr[i].flush();
			imageArr[i] = null;
		}
		mt = null;
		System.gc();
		System.runFinalization();
		
		
	}
	public void saveMBSFile(Vector<Layer> layerVector){
		Vector<Layer> vector = layerVector;
		JFileChooser chooser = new JFileChooser(); 
		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		chooser.showSaveDialog(null);
		File file = chooser.getSelectedFile();
		FileOutputStream fout;
		ObjectOutputStream os;
		
		try {
			fout = new FileOutputStream(file);
			os = new ObjectOutputStream(fout);
			os.writeObject(vector);
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}