package panels;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import java.util.Vector;

import javax.swing.JPanel;

import popupMenus.PopupSelction;
import utils.DrawStateSingleton;
import utils.DrawableInformation;
import utils.Storage;
import Layers.Layer;
import Shapes.ElementImage;
import Shapes.ParentElement;
import Start.MainFrame;

public class CanvasPanel extends JPanel{

	private static final long serialVersionUID = 1L;
	public MainFrame mainframe;
	private int nwback , nhback;
	private int  fontSize=30;
	private Storage storage;
	private String fontType = new String("TimeRoman");
	public Image gBackImg , currentSelectedImage;
	public Graphics2D gback=null;
	private DrawableInformation info;
	public  Vector<Layer> layerVector;
	public Layer currentLayer;
	public Vector<ParentElement> selectVector;
	public PopupSelction popup;
	public Vector<ParentElement> undoVector;
	
	public CanvasPanel(MainFrame mainframe) {
		super();
		
		this.mainframe = mainframe;
		selectVector = new   Vector<ParentElement>();
		undoVector = new   Vector<ParentElement>();
		info = new DrawableInformation();
		currentLayer = new Layer(this);
		layerVector = new Vector<Layer>();
		popup = new PopupSelction(this);
		layerVector.add(currentLayer);
		this.sub();
	}	
	
	public void sub() {
		storage = new Storage();
		this.info.setFill(true);
		this.info.setLineColor(Color.black);
		this.info.setFillColor(Color.yellow);
		this.info.setCanvasSize(this.getWidth(),this.getHeight());
		this.setComponentPopupMenu(this.popup);
	
		updateDrawState();
		this.setBackground(Color.white);
       
	}
	
	public void updateDrawState(){
		if(this.getMouseListeners().length>0){
			this.removeMouseListener(this.getMouseListeners()[0]);
			this.removeMouseMotionListener(this.getMouseMotionListeners()[0]);
		}
		
		this.addMouseListener(DrawStateSingleton.getInstance().getToolState());
		this.addMouseMotionListener(DrawStateSingleton.getInstance().getToolState());
		System.gc();
		System.runFinalization();
	}
	
	public void removeDrawState(){
		if(this.getMouseListeners().length>0){
			this.removeMouseListener(this.getMouseListeners()[0]);
			this.removeMouseMotionListener(this.getMouseMotionListeners()[0]);
		}
		
	}
	
	public void setImageToCanvasPanel(Image temporeryImage) {
		if(temporeryImage == null){}
		else{this.currentSelectedImage = temporeryImage;}				
	}
	
	public DrawableInformation getInformation(){
		return this.info;
	}
	public void setStrokeSize(int strokeSize){
		this.info.setStrokeSize(strokeSize);
	}
	public int getStrokeSize(){
		return info.getStrokeSize();
	}
	public void setFontSize(int fontSize){
		this.fontSize = fontSize;
	}
	public int getFontSize(){
		return this.fontSize;
	}
	public void setFontType(String fontType){
		this.fontType = fontType;
	}
	public String getFontType(){
		return this.fontType;
	}
	
	public void setLineColor(Color lineColor){
		this.info.setLineColor(lineColor);
	}
	public void setFillColor(Color fillColor){
		this.info.setFillColor(fillColor);
	}
	
	public void setFill(boolean isFilled){
		this.info.setFill(isFilled);
	}
	public void setTexture(boolean isTexture){
		this.info.setTexture(isTexture);
	}
	public void setAntiAlias(boolean antiAlias){
		this.info.setAntiAlias(antiAlias);
	}
	public void setAlphaInterpolation(boolean alphaInterpolation){
		this.info.setAlphaInterpolation(alphaInterpolation);
	}
	
	public void setColorRendering(boolean colorRendering){
		this.info.setColorRendering(colorRendering);
	}
	
	public void setDithering(boolean dithering){
		this.info.setDithering(dithering);
	}
	
	public void setFractionalMetrics(boolean fractionalMetrics){
		this.info.setFractionalMetrics(fractionalMetrics);
	}
	
	public void setRendering(boolean rendering){
		this.info.setRendering(rendering);
	}
	
	public void setTextAntiAliasing(boolean textAntiAliasing){
		this.info.setTextAntiAliasing(textAntiAliasing);
	}
	


	public void update(){
		if(gback != null){
		gback.setBackground(Color.white);
		gback.clearRect(0, 0, nwback-1, nhback-1);/*clearRect ϸ Ʈ ȸ*/ 
		for(int i=0;i<storage.getDrawableList().size();i++){
			if(storage.getDrawableList().get(i).isVisible){
				storage.getDrawableList().get(i).paint(gback);
			}
		}
		}
	}
	public void swapBuffer(){
		Graphics g =this.getGraphics();
		if(gBackImg != null & g != null){
			g.drawImage(gBackImg, 0, 0, this);
		}
	}
	public void paint(java.awt.Graphics g){
		super.paint(g);	
    	update(g);
	}
	
	public void update(Graphics g){

		Graphics2D g2 = (Graphics2D)g;
    	
		int w = this.getWidth();
    	int h = this.getHeight();
    	if((gBackImg == null ) || (this.nwback != w) || (this.nhback != h)){
    		if(gBackImg != null){
    			gBackImg.flush();  // ̹ ҽ 
    		}
    		gBackImg = this.createImage(w, h);
    		nwback = w; nhback = h;
    	}
    	gback = (Graphics2D)gBackImg.getGraphics();
		gback.setBackground(Color.white);
		gback.clearRect(0, 0, nwback, nhback);   
    	
        for(int i = 0 ; i < layerVector.size(); i++) {
        	if(layerVector.elementAt(i).isVisible == true){
        		this.layerVector.elementAt(i).paint();
        		AlphaComposite ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 
        				this.layerVector.elementAt(i).opacity);
        		gback.setComposite(ac);
        		gback.drawImage(layerVector.elementAt(i).buffer, 0, 0, this);
        		ac = AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 
        				1.0f);
        		gback.setComposite(ac);
        	}
        }
        for(int i = 0 ;i<this.selectVector.size();i++){
        	this.selectVector.elementAt(i).paint(gback);
        }
        
        g2.drawImage(gBackImg, 0, 0, this);
       
	}
	
	public void addElement(ParentElement element) {
		this.mainframe.addElement(element);
	}
	public void updateElement(){
		this.mainframe.updateElement(storage.getDrawableList());
		
	}

	public Storage getStorage(){
		return this.storage;
	}
	
	public void showPopUp(int x, int y){
		this.popup.show(this, x,y);
	}
	
	public void addLayer(){
		Layer l = new Layer(this);
		this.layerVector.add(l);
		this.currentLayer = l;
		this.repaint();
		this.updateElement();
		
	}
	
	public void copy(int i){
		
		if(this.selectVector.elementAt(i).getShape() != null){
			Rectangle rect = new Rectangle(this.selectVector.elementAt(i).getShape().getBounds());
			BufferedImage img = new BufferedImage(this.getWidth(),this.getHeight(),BufferedImage.TYPE_4BYTE_ABGR);
			Graphics2D g2d = img.createGraphics();
			g2d.setClip(this.selectVector.elementAt(i).getShape());
			
			Image ig = this.createImage(this.gBackImg.getWidth(this), this.gBackImg.getHeight(this));
			Graphics2D gg = (Graphics2D) ig.getGraphics();
			for(int j = 0 ; j < layerVector.size(); j++) {
	        	if(layerVector.elementAt(j).isVisible == true){
	        		this.layerVector.elementAt(j).paint();
	        		gg.drawImage(layerVector.elementAt(j).buffer, 0, 0, this);
	        	}
	        }
			g2d.drawImage(ig, 0, 0, null);
			
			BufferedImage sub = img.getSubimage(rect.x, rect.y, rect.width, rect.height);
			DrawableInformation in =this.info.makeClone();
			in.setCanvasSize(this.getWidth(),this.getHeight());
			ElementImage ele = new ElementImage(in , null);
			ele.setImage(sub);
			ele.setRect(rect);
			
			if(this.selectVector.elementAt(i).getShape() instanceof GeneralPath){  // bright contrast grad Ҷ ʿ
				GeneralPath g = new GeneralPath(this.selectVector.elementAt(i).getShape());
				ele.setShape(g);
			}else{
				ele.setShape(rect);
			}
			
			
			this.mainframe.setSelectedElement(ele);
		
		}
		  
	}
	public void paste(int x, int y){
		if(this.mainframe.getSelectedElement() != null){
			ParentElement ele = mainframe.getSelectedElement();
			ele.setStartPoint(new Point(x,y));
			this.getStorage().getDrawableList().add(ele);
			this.currentLayer.addElement(ele);
			ele.setLayer(this.currentLayer);
			ele.information.setCanvasSize(this.getWidth(), this.getHeight());
			this.repaint();
			this.updateElement();
		}
	}
	
	public void removeElement(ParentElement ele){

		storage.getDrawableList().remove(ele);
		this.undoVector.add(ele);
		for(int i=0;i< this.layerVector.size();i++){
			for(int j=0;j<layerVector.elementAt(i).vector.size();j++){
				if(ele == layerVector.elementAt(i).vector.elementAt(j)){
					layerVector.elementAt(i).vector.remove(ele);
					repaint();
					updateElement();
				}
			}
		}
	
	}
	
	public void removeElement(){
		if(this.storage.getDrawableList().size() != 0){
			ParentElement ele = storage.getDrawableList().get(storage.getDrawableList().size()-1);
			storage.getDrawableList().remove(storage.getDrawableList().size()-1);
			this.undoVector.add(ele);
			for(int i=0;i<this.layerVector.size();i++){
				for(int j=0; j< this.layerVector.elementAt(i).vector.size();j++){
					if(ele == this.layerVector.elementAt(i).vector.elementAt(j)){
						this.layerVector.elementAt(i).vector.remove(j);
						this.repaint();
						this.updateElement();
						break;
					}
				}
			}
		}
	}
	public void redoElement(){
		if(this.undoVector.size()!= 0){
			this.storage.getDrawableList().add(this.undoVector.lastElement());
			for(int i=0;i<this.layerVector.size();i++){
				if(this.layerVector.elementAt(i) == this.undoVector.lastElement().getLayer()){
					this.layerVector.elementAt(i).vector.add(this.undoVector.lastElement());
					this.undoVector.remove(undoVector.size()-1);
					this.repaint();
					this.updateElement();
					
					return;
				}
			}
			
			
			
			this.currentLayer.vector.add(this.undoVector.lastElement());
			this.undoVector.lastElement().setLayer(this.currentLayer);
			this.undoVector.remove(undoVector.size()-1);
			this.repaint();
			this.updateElement();
			return;
		}
	}
	
	public void removeLayer(Layer layer){
		for(int i = 0 ;i< layer.vector.size();i++){
			for(int j=0;j< this.storage.getDrawableList().size();j++){
				if(layer.vector.elementAt(i) == storage.getDrawableList().get(j)){
					storage.getDrawableList().remove(j);
				}
			}
		}
		this.layerVector.remove(layer);
		
		if(this.layerVector.size() == 0){
			this.addLayer();
		}
		if(layer == this.currentLayer ){
			this.currentLayer = this.layerVector.lastElement();
		}
		this.repaint();
		this.updateElement();
	}
}
