package panels;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.GradientPaint;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.color.ColorSpace;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ColorConvertOp;
import java.awt.image.ConvolveOp;
import java.awt.image.Kernel;
import java.awt.image.LookupOp;
import java.awt.image.RescaleOp;
import java.awt.image.ShortLookupTable;
import java.util.Hashtable;

import javax.swing.JPanel;

import Shapes.ElementImage;

public class DisplayPanel extends JPanel{
		
		private static final long serialVersionUID = 918047255786817293L;
		
		public BufferedImage img ,dst ,repalce;
		public boolean blur= false,sharpen= false,edge= false,gray = false,
		posterize = false,invert = false,bright = false,contrast = false;
		public boolean rInvert= false,rRemove= false;
		public boolean gInvert= false,gRemove= false;
		public boolean bInvert= false,bRemove= false;
		private Hashtable<String, BufferedImageOp> hash;
		private ElementImage element;
		private RescaleOp rescale;
		public boolean grad;
		public float gradOpacity;
		public int gradTheta;
		public Color lineColor;
		public Color fillColor;
		public DisplayPanel(ElementImage element){
			
			this.element = element;
			this.fillColor = this.element.information.getFillColor();
			this.lineColor = this.element.information.getLineColor();
			//System.out.println(this.element.information.getFillColor());
			hash = new Hashtable<String, BufferedImageOp>();
			this.createBufferedImages();
			createConvolutions();
			createLookups();
			createColorOps();
		}
		
		public void setImage(BufferedImage i){
			this.repalce = i;
		}
		
		public void createBufferedImages() {
			//img = new BufferedImage(element.rect.width, //sourcce buffered img
	        //                          element.rect.height,
	        //                          BufferedImage.TYPE_4BYTE_ABGR);
			img = new BufferedImage(250, //sourcce buffered img
			                                  200,
			                                  BufferedImage.TYPE_4BYTE_ABGR);
			Graphics2D big = img.createGraphics();
	        //big.drawImage(element.bufferedImage, 0, 0,element.rect.width, //sourcce buffered img
	        //        element.rect.height, this);
			big.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
	        big.drawImage(element.bufferedImage, 0, 0,250, //sourcce buffered img
	                200, this);
	     
	        
	       // dst = new BufferedImage(element.rect.width, //dest buffered img
	       // 						   element.rect.height,
	       // 						   BufferedImage.TYPE_4BYTE_ABGR);
	        						   
		   dst = new BufferedImage(250, //dest buffered img
				   200,
				   BufferedImage.TYPE_4BYTE_ABGR);
	        Graphics2D bg = dst.createGraphics();
	       // bg.drawImage(element.bufferedImage, 0, 0,element.rect.width, //sourcce buffered img
	       //         element.rect.height, this);
	        bg.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
	        bg.drawImage(element.bufferedImage, 0, 0,250, //sourcce buffered img
	     	                200, this);

		}		
		
		public void createConvolutions() {
		    float ninth = 1.0f / 9.0f;
		    float[] blurKernel = { ninth, ninth, ninth, ninth, ninth, ninth, ninth,
		        ninth, ninth };
		    hash.put("Blur", new ConvolveOp(new Kernel(3, 3, blurKernel),
		        ConvolveOp.EDGE_NO_OP, null));
		    
		    float[] edge = { 0f, -1f, 0f, -1f, 4f, -1f, 0f, -1f, 0f };
		    hash.put("Edge detector", new ConvolveOp(new Kernel(3, 3, edge),
		        ConvolveOp.EDGE_NO_OP, null));

		    float[] sharp = { 0f, -1f, 0f, -1f, 5f, -1f, 0f, -1f, 0f };
		    hash.put("Sharpen", new ConvolveOp(new Kernel(3, 3, sharp)));
		}
		
		private void createColorOps() {
			hash.put("Grayscale", new ColorConvertOp(ColorSpace
		        .getInstance(ColorSpace.CS_GRAY), null));
		}
		
		private void createLookups() {
		    short[] brighten = new short[256];
		    short[] betterBrighten = new short[256];
		    short[] posterize = new short[256];
		    short[] invert = new short[256];
		    short[] straight = new short[256];
		    short[] zero = new short[256];

		    for (int i = 0; i < 256; i++) {
		      brighten[i] = (short) (128 + i / 2);
		      betterBrighten[i] = (short) (Math.sqrt((double) i / 255.0) * 255.0);
		      posterize[i] = (short) (i - (i % 32));
		      invert[i] = (short) (255 - i);
		      straight[i] = (short) i;
		      zero[i] = (short) 0;
		    }
		    
		    hash.put("Brighten", new LookupOp(new ShortLookupTable(0, brighten),
		        null));
		    hash.put("Better Brighten", new LookupOp(new ShortLookupTable(0,
		        betterBrighten), null));
		    hash.put("Posterize", new LookupOp(new ShortLookupTable(0, posterize),
		        null));
		    hash.put("Invert", new LookupOp(new ShortLookupTable(0, invert), null));

		    short[][] redOnly = { invert, straight, straight };
		    short[][] greenOnly = { straight, invert, straight };
		    short[][] blueOnly = { straight, straight, invert };
		    
		    hash.put("Red invert", new LookupOp(new ShortLookupTable(0, redOnly),
		        null));
		    hash.put("Green invert", new LookupOp(
		        new ShortLookupTable(0, greenOnly), null));
		    hash.put("Blue invert", new LookupOp(new ShortLookupTable(0, blueOnly),
		        null));

		    short[][] redRemove = { zero, straight, straight };
		    short[][] greenRemove = { straight, zero, straight };
		    short[][] blueRemove = { straight, straight, zero };
		    
		    hash.put("Red remove", new LookupOp(new ShortLookupTable(0, redRemove),
		        null));
		    hash.put("Green remove", new LookupOp(new ShortLookupTable(0,
		        greenRemove), null));
		    hash.put("Blue remove", new LookupOp(
		        new ShortLookupTable(0, blueRemove), null));
		}
		
		public void brightRescale(int offset){
			int r = (offset-50)*5;
			rescale = new RescaleOp(1.0f, r, null);
		}
		
		public void ContrastRescale(float offset){
			float r = offset/10;
			rescale = new RescaleOp(r, 10, null);
		}
		
		public void paint(Graphics g){
			super.paint(g);
			Graphics2D g2d = (Graphics2D)g;
			this.createBufferedImages();
			if(this.repalce != null){
				dst = this.repalce;
			}
			if(this.blur == true){
				BufferedImageOp imageOp = (BufferedImageOp) hash.get("Blur");
				imageOp.filter(img, dst);
			}
			if(this.sharpen == true){
				BufferedImageOp imageOp = (BufferedImageOp) hash.get("Sharpen");
				imageOp.filter(img, dst);
			}
			if(this.edge == true){
				BufferedImageOp imageOp = (BufferedImageOp) hash.get("Edge detector");
				imageOp.filter(img, dst);
			}			
			if(this.posterize == true){
				BufferedImageOp imageOp = (BufferedImageOp) hash.get("Posterize");
				imageOp.filter(dst, dst);
			}
			if(this.gray == true){
				BufferedImageOp imageOp = (BufferedImageOp) hash.get("Grayscale");
				imageOp.filter(dst, dst);
			}
			if(this.invert == true){
				BufferedImageOp imageOp = (BufferedImageOp) hash.get("Invert");
				imageOp.filter(dst, dst);
			}
			
			if(this.bright == true){
				
				BufferedImage bi = new BufferedImage(250,200,BufferedImage.TYPE_INT_RGB);
				Graphics2D big = bi.createGraphics();
				big.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
				big.drawImage(dst, 0, 0, null);
				rescale.filter(bi, bi);
				Graphics2D dstg = dst.createGraphics();
				
				
				if(this.element.getShape()!= null){
					if(this.element.getShape() instanceof GeneralPath ){
						
						dstg.setClip(null);
						Rectangle r = ((GeneralPath)this.element.getShape()).getBounds();
						
						AffineTransform Tx = AffineTransform.getTranslateInstance(
								-r.x, -r.y);
						((GeneralPath)this.element.getShape()).transform(Tx);
						float width = (float)250/(float)r.width;
						float height = (float)200/(float)r.height;
						
						AffineTransform x = AffineTransform.getScaleInstance(width, height);
						((GeneralPath)this.element.getShape()).transform(x);
						dstg.clip(this.element.getShape());
						dstg.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
						dstg.drawImage(bi, 0, 0, null);
						
						Tx = AffineTransform.getTranslateInstance(       // ٽ  ġ  ǵ.
								r.x, r.y);
						((GeneralPath)this.element.getShape()).transform(Tx);
						x = AffineTransform.getScaleInstance(1/width, 1/height);
						
					}else{
						dstg.setClip(null);
						Rectangle r = ((Rectangle)this.element.getShape()).getBounds();
						int rw  = r.width;
						int rh = r.height;
						((Rectangle)this.element.getShape()).translate(-r.x, -r.y);
						
						((Rectangle)this.element.getShape()).width = 250;
						((Rectangle)this.element.getShape()).height = 200;
						
						dstg.clip(this.element.getShape());
						dstg.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
						dstg.drawImage(bi, 0, 0, null);
						
						((Rectangle)this.element.getShape()).translate(r.x, r.y);
						((Rectangle)this.element.getShape()).width = rw;
						((Rectangle)this.element.getShape()).height = rh;
						
					}
				}else{
					dstg.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
					dstg.drawImage(bi, 0, 0, null);
				}
			}
			if(this.contrast == true){
				
				BufferedImage bi = new BufferedImage(250,200,BufferedImage.TYPE_INT_RGB);
				Graphics2D big = bi.createGraphics();
				big.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
				big.drawImage(dst, 0, 0, null);
				rescale.filter(bi, bi);
				Graphics2D dstg = dst.createGraphics();
				
				if(this.element.getShape()!= null){
					if(this.element.getShape() instanceof GeneralPath ){
						
						dstg.setClip(null);
						Rectangle r = ((GeneralPath)this.element.getShape()).getBounds();
						
						AffineTransform Tx = AffineTransform.getTranslateInstance(
								-r.x, -r.y);
						((GeneralPath)this.element.getShape()).transform(Tx);
						float width = (float)250/(float)r.width;
						float height = (float)200/(float)r.height;
						
						AffineTransform x = AffineTransform.getScaleInstance(width, height);
						((GeneralPath)this.element.getShape()).transform(x);
						dstg.clip(this.element.getShape());
						dstg.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
						dstg.drawImage(bi, 0, 0, null);
						
						Tx = AffineTransform.getTranslateInstance(       // ٽ  ġ  ǵ.
								r.x, r.y);
						((GeneralPath)this.element.getShape()).transform(Tx);
						x = AffineTransform.getScaleInstance(1/width, 1/height);
						
					}else{
						dstg.setClip(null);
						Rectangle r = ((Rectangle)this.element.getShape()).getBounds();
						int rw  = r.width;
						int rh = r.height;
						((Rectangle)this.element.getShape()).translate(-r.x, -r.y);
						
						((Rectangle)this.element.getShape()).width = 250;
						((Rectangle)this.element.getShape()).height = 200;
						
						dstg.clip(this.element.getShape());
						dstg.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
						dstg.drawImage(bi, 0, 0, null);
						
						((Rectangle)this.element.getShape()).translate(r.x, r.y);
						((Rectangle)this.element.getShape()).width = rw;
						((Rectangle)this.element.getShape()).height = rh;
						
					}
				}else{
					dstg.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
					dstg.drawImage(bi, 0, 0, null);
				}
			}
			
			Graphics2D gg= dst.createGraphics();
			gg.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
			gg.drawImage(dst, 0, 0, null);
			if(this.grad == true){
				Rectangle rect = new Rectangle(0,0,this.dst.getWidth(),this.dst.getHeight());
				AlphaComposite c = AlphaComposite.getInstance(
						AlphaComposite.SRC_OVER, this.gradOpacity);
				gg.setComposite(c);
				//System.out.println("here");
				if(this.gradTheta !=0) {
					
					gg.clip(rect);
					if(this.element.getShape()!= null){ // copy Ҷ setShape ֹǷ ߶  ƴϸ null ߶ ̸ null  ƴϴ. 
						if(this.element.getShape() instanceof GeneralPath){
							gg.setClip(null);
							Rectangle r = ((GeneralPath)this.element.getShape()).getBounds();
							//System.out.println(r);
							
							AffineTransform Tx = AffineTransform.getTranslateInstance(
									-r.x, -r.y);
							((GeneralPath)this.element.getShape()).transform(Tx);
							
							float width = this.getWidth()/r.width;
							float height = this.getHeight()/r.height;
						
						
							AffineTransform x = AffineTransform.getScaleInstance(width, height);
							((GeneralPath)this.element.getShape()).transform(x);
							gg.clip(this.element.getShape());
						
						}
					}
					float cross = (rect.width)*(rect.width) + (rect.height)*(rect.height);
					int result = (int)Math.sqrt(cross);
					Rectangle rect2 = new Rectangle( (int)(- result/2) , (int)(- result /2) ,result,result);
					AffineTransform at2 = new AffineTransform();	
					AffineTransform saveform = gg.getTransform();		
					at2.setToTranslation(rect.x + rect.width/2, rect.y + rect.height/2);
					at2.rotate(Math.toRadians(this.gradTheta)); 
					gg.transform(at2); 
					GradientPaint grad;
			  		grad=new GradientPaint( rect2.x , rect2.y,
			  				this.lineColor,rect2.x+rect2.width,
			  				rect2.y + rect2.height ,this.fillColor, false);
			  		gg.setPaint(grad);
			  		gg.fill(rect2);
			  		//System.out.println("here");
			  		gg.setTransform(saveform);
			  		gg.setClip(null);
				}else{
					gg.clip(rect);
					if(this.element.getShape()!= null){
						if(this.element.getShape() instanceof GeneralPath){
							gg.setClip(null);
							Rectangle r = ((GeneralPath)this.element.getShape()).getBounds();
							
							AffineTransform Tx = AffineTransform.getTranslateInstance(
									-r.x, -r.y);
							((GeneralPath)this.element.getShape()).transform(Tx);
							float width = (float)250/(float)r.width;
							float height = (float)200/(float)r.height;
							//System.out.println("width :"+width+"height :"+height);
							//System.out.println(r);
							AffineTransform x = AffineTransform.getScaleInstance(width, height);
							((GeneralPath)this.element.getShape()).transform(x);
							gg.clip(this.element.getShape());
							
						}
					}
					GradientPaint grad;
			  		grad=new GradientPaint( rect.x,rect.y,
			  				this.lineColor,rect.x+rect.width,
			  				rect.y+rect.height ,this.fillColor, false);
			  		gg.setPaint(grad);
			  		gg.fill(rect);
				}
				
			}
			g2d.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC);
			g2d.drawImage(dst, 2, 2,this.getWidth()-5,this.getHeight()-5, this);
		}
	}	