package utils;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.io.Serializable;



public class DrawableInformation implements Serializable{
	private static final long serialVersionUID = 1L;
	/*Attributes*/
	public float gradTheta =0f;
	public int strokeSize = 1 ;
	private float rotationTheta;
	public Color lineColor , fillColor ;
	public boolean isFilled = false;
	private boolean isTexture = false;
	private boolean isGradation = false;
	public boolean isRoatated = false;
	private Point canvasSize = new Point();
	private Point gradStartPoint = new Point(0,0);
	private Point gradEndPoint = new Point(600,600);
	/*Rendering Attributes*/
	private boolean antiAlias = true;
	private boolean alphaInterpolation = false;
	private boolean colorRendering = false;
	private boolean dithering = false;
	private boolean fractionalMetrics = false;
	private boolean rendering = false;
	private boolean textAntiAliasing = false;
	public float opacity = 1.0f ;
	public float gradOpacity = 1.0f;
	
	public DrawableInformation(){		
		
	}
	
	/*Duplicate*/
	public DrawableInformation(DrawableInformation inform){
		DrawableInformation df = new DrawableInformation();
		
		df.alphaInterpolation = inform.alphaInterpolation;
		df.antiAlias = inform.antiAlias;
		df.colorRendering = inform.colorRendering;
		df.dithering = inform.dithering;
		df.fillColor = inform.fillColor;
		df.fractionalMetrics = inform.fractionalMetrics;
		df.isFilled = inform.isFilled;
		df.isGradation= inform.isGradation;
		df.isTexture = inform.isTexture; 
		df.lineColor = inform.lineColor; //
		df.rendering = inform.rendering;
		df.strokeSize = inform.strokeSize; //
		df.textAntiAliasing = inform.textAntiAliasing;
	}
	
	/*Clone*/
	public DrawableInformation makeClone(){
		DrawableInformation df = new DrawableInformation();
		
		df.alphaInterpolation = this.alphaInterpolation;
		df.antiAlias = this.antiAlias;
		df.colorRendering = this.colorRendering;
		df.dithering = this.dithering;
		df.fillColor = this.fillColor;
		df.fractionalMetrics = this.fractionalMetrics;
		df.isFilled = this.isFilled;
		df.isGradation= this.isGradation;
		df.isTexture = this.isTexture;
		//df.canvasSize.x = this.canvasSize.x;
		//df.canvasSize.y = this.canvasSize.y;
		df.setCanvasSize(this.canvasSize.x,this.canvasSize.y);
		df.setLineColor(this.lineColor);
		df.rendering = this.rendering;
		df.strokeSize = this.strokeSize;
		df.textAntiAliasing = this.textAntiAliasing;
		df.isRoatated = this.isRoatated;
		df.opacity = this.opacity;
		df.gradOpacity = this.gradOpacity;
		return df;
	}
	
	public void setGraphicAttribute(Graphics2D g){
		g.setStroke(new BasicStroke(this.strokeSize,1,0));
		g.setColor(this.lineColor);
		
		
		if(this.isTexture){
			
		}
		if(this.antiAlias){
			g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		}
		if(this.alphaInterpolation){
			g.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
		}else{
			g.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_DEFAULT);
		}
		if(this.colorRendering){
			g.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY);
		}else{
			g.setRenderingHint(RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_DEFAULT);
		}
		if(this.dithering){
			g.setRenderingHint(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_ENABLE);
		}else{
			g.setRenderingHint(RenderingHints.KEY_DITHERING, RenderingHints.VALUE_DITHER_DEFAULT);
		}
		if(fractionalMetrics){
			g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_ON);
		}else{
			g.setRenderingHint(RenderingHints.KEY_FRACTIONALMETRICS, RenderingHints.VALUE_FRACTIONALMETRICS_DEFAULT);
		}
		if(rendering){
			g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		}else{
			g.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_DEFAULT);
		}
		if(textAntiAliasing){
			g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		}else{
			g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_OFF);
		}
	
	}
	public void setGradStart(Point p){
		this.gradStartPoint = (Point)p;
	}

	public Point getGradStart(){
		return this.gradStartPoint;
	}
	public void setGradEnd(Point p){
		this.gradEndPoint = (Point)p;
	}

	public Point getGradEnd(){
		return this.gradEndPoint;
	}
	public void setRotationTheta(float theta){
		this.rotationTheta = theta;
	}
	public float getRotationTheta(){
		return this.rotationTheta;
	}
	public void setCanvasSize(int x,int y){
		this.canvasSize.x = x;
		this.canvasSize.y = y;
	}

	public Point getCanvasSize(){
		return this.canvasSize;
	}
	public void setStrokeSize(int strokeSize){
		this.strokeSize = strokeSize;
	}
	public int getStrokeSize(){
		return this.strokeSize;
	}
	
	public void setLineColor(Color lineColor){
		this.lineColor = lineColor;
	}
	public Color getLineColor(){
		return this.lineColor;
	}
	
	public void setFillColor(Color fillColor){
		this.fillColor = fillColor;
	}
	public Color getFillColor(){
		return this.fillColor;
	}
	
	public void setFill(boolean isFilled){
		this.isFilled = isFilled;
	}
	public boolean getFill(){
		return this.isFilled;
	}
	
	public void setGradation(boolean isGradation){
		this.isGradation = isGradation;
	}
	public boolean getGradation(){
		return this.isGradation;
	}
	
	public void setTexture(boolean isTexture){
		this.isTexture = isTexture;
	}
	public boolean getTexture(){
		return this.isTexture;
	}
	
	public void setAntiAlias(boolean antiAlias){
		this.antiAlias = antiAlias;
	}
	public boolean getAntiAlias(){
		return this.antiAlias;
	}
	
	public void setAlphaInterpolation(boolean alphaInterpolation){
		this.alphaInterpolation = alphaInterpolation;
	}
	public boolean gettAlphaInterpolation(){
		return this.alphaInterpolation ;
	}
	
	public void setColorRendering(boolean colorRendering){
		this.colorRendering = colorRendering;
	}
	public boolean  getColorRendering(){
		return this.colorRendering;
	}
	
	public void setDithering(boolean dithering){
		this.dithering = dithering;
	}
	public boolean getDithering(){
		return this.dithering;
	}
	
	public void setFractionalMetrics(boolean fractionalMetrics){
		this.fractionalMetrics = fractionalMetrics;
	}
	public boolean getFractionalMetrics(){
		return this.fractionalMetrics;
	}
	
	public void setRendering(boolean rendering){
		this.rendering = rendering;
	}
	public boolean getRendering(){
		return this.rendering;
	}
	
	public void setTextAntiAliasing(boolean textAntiAliasing){
		this.textAntiAliasing = textAntiAliasing;
	}
	public boolean setTextAntiAliasing(){
		return this.textAntiAliasing;
	}
	
}
